;; Lobster - Copyright 2018 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Hook up 2 motors, 2 servos, and an IR sensor to a lobster to make
;; it a remote controlled lobster.

.include "tn85def.inc"
.avr8

; CKSEL = 0001
; CLKPS = 0000
; CKDIV8 = 0
;
; FUSE LOW = 11100001 = 0xe1

;  cycles  sample rate   @16 MHz:
;    210   38.0kHz * 2

; HH YAW PTCH THROTTLE YAW_CORRECT
; YAW = 0-126  63
; PITCH = 0-126 63
; THROTTLE = 0-63  (bit 7 is A/B switch)
; CORR = 0-126 63

; r0  = 0
; r1  = 1
; r15 = 255
; r14 = temp
; r17 = temp
; r18 = claw high / low value (counter)
; r20 = count in interrupt
; r21 = current byte being read in
; r22 = byte_count
; r23 = bit_count
; r24 = servo 1 position
; r25 = servo 0 position
; r26 = idle counter low
; r27 = idle counter high
; r28 = servo interrupt count low
; r29 = servo interrupt count low
; r30 = Z
; r31 = Z (pointer to IR data)
;

; note: CLKSEL 10

IR_DATA equ SRAM_START
YAW equ IR_DATA + 0
PITCH equ IR_DATA + 1
THROTTLE equ IR_DATA + 2
YAW_CORRECTION equ IR_DATA + 3
SERVO_VALUE_0 equ IR_DATA + 4
SERVO_VALUE_1 equ IR_DATA + 5

ROTATE_LOW equ 100
ROTATE_MIDDLE equ 114
ROTATE_HIGH equ 128

.org 0x000
  rjmp start
.org 0x00a
  rjmp service_interrupt

;; FIXME - erase this padding..
.org 0x020

start:
  ;; Disable interrupts
  cli

  ;; Set up stack ptr
  ;ldi r17, RAMEND>>8
  ;out SPH, r17
  ldi r17, RAMEND & 255
  out SPL, r17

  ;; r0 = 0, r1 = 1, r15 = 255
  eor r0, r0
  eor r1, r1
  inc r1
  ldi r17, 0xff
  mov r15, r17

  ;; Set up PORTB
  ;; PB0: IR input
  ;; PB1: Servo 0
  ;; PB2: Motor 0
  ;; PB3: Motor 1
  ;; PB4: Servo 1
  ldi r17, 0x1e
  out DDRB, r17             ; entire port B is output except PB0
  out PORTB, r0             ; turn off all of port B

  ;; Thrust is initialized to 0, idle counter is at 0
  ldi r26, 0
  ldi r27, 0
  ldi r28, 0xf0
  ldi r29, 0x05

  ldi r17, 240 - ROTATE_LOW
  sts SERVO_VALUE_0, r17
  ldi r17, 240 - ROTATE_HIGH
  sts SERVO_VALUE_1, r17

  ;; Set up TIMER0
  ldi r17,  210                  ; set carrier freq
  out OCR0A, r17

  ldi r17, (1<<OCIE0A)
  out TIMSK, r17                 ; enable interrupt compare A 
  ldi r17, (1<<WGM01)
  out TCCR0A, r17                ; normal counting (0xffff is top, count up)
  ldi r17, (1<<CS00)             ; CTC OCR0A  Clear Timer on Compare
  out TCCR0B, r17                ; prescale = 1 from clock source

  ; Enable interrupts
  sei

  rcall delay
  rcall delay
  rcall delay

main:
  rcall ir_comm_read

  ldi r26, 0
  ldi r27, 0

  lds r17, THROTTLE
  andi r17, 0x7f   ; filter out A/B switch
  cpi r17, 0x38
  brlo motors_off

  lds r17, PITCH

  cpi r17, 0x20
  brsh not_forward
  inc r18
  sbrs r18, 3
  ldi r17, 240 - ROTATE_HIGH
  sbrc r18, 3
  ldi r17, 240 - ROTATE_LOW
  sts SERVO_VALUE_0, r17
  sts SERVO_VALUE_1, r17
  sbi PORTB, 2
  sbi PORTB, 3
  rjmp main

not_forward:
  lds r17, YAW

  cpi r17, 0x20
  brsh not_left
  cbi PORTB, 2
  sbi PORTB, 3
  ldi r17, 240 - ROTATE_LOW
  sts SERVO_VALUE_0, r17
  sts SERVO_VALUE_1, r17
  rjmp main

not_left:
  cpi r17, 0x50
  brlo not_right
  sbi PORTB, 2
  cbi PORTB, 3
  ldi r17, 240 - ROTATE_HIGH
  sts SERVO_VALUE_0, r17
  sts SERVO_VALUE_1, r17
  rjmp main

not_right:
motors_off:
  cbi PORTB, 2
  cbi PORTB, 3
  ldi r17, 240 - ROTATE_LOW
  sts SERVO_VALUE_0, r17
  ldi r17, 240 - ROTATE_HIGH
  sts SERVO_VALUE_1, r17
  rjmp main

.include "ir_comm.asm"

delay:
  ldi r17, 100
delay_repeat_outer:
  ldi r20, 0
delay_repeat:
  cpi r20, 255
  brne delay_repeat
  dec r17
  brne delay_repeat_outer
  ret

service_interrupt:
  in r7, SREG
  inc r20

  ;; Service servos
  sbiw r28, 1
  brne not_20ms
  ldi r28, 0xf0
  ldi r29, 0x05
  sbi PORTB, 1
  sbi PORTB, 4
  lds r24, SERVO_VALUE_1
  lds r25, SERVO_VALUE_0
not_20ms:
  cp r28, r25
  brne dont_update_servo_0
  cbi PORTB, 1
dont_update_servo_0:
  cp r28, r24
  brne dont_update_servo_1
  cbi PORTB, 4
dont_update_servo_1:

  ;; Count how long there is no IR data.  For safety, if there is no
  ;; for around 1 second, then reset the motors.
  ;dec r26
  ;brne exit_idle_counter
  ;dec r27
  ;brne exit_idle_counter
  sbiw r26, 1
  brne exit_idle_counter
  cbi PORTB, 2
  cbi PORTB, 3
exit_idle_counter:

exit_interrupt:
  out SREG, r7
  reti

